@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -425,7 +425,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                     final IMAPStoreCache imapStoreCache = IMAPStoreCache.getInstance();
                     if (null == imapStoreCache) {
                         closeSafely(imapStore);
-                    } else if (imapStore.isConnectedUnsafe()) {
+                    } else if (imapStore.isConnected()) {
                         imapStoreCache.returnIMAPStore(imapStore, accountId, server, port, login, session);
                     } else {
                         // Not null AND not connected...?
@@ -602,6 +602,16 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 warnings.add(MailExceptionCode.PING_FAILED_AUTH.create(e, config.getServer(), config.getLogin()));
                 throw MimeMailException.handleMessagingException(e, config, session);
             } catch (final MessagingException e) {
+                Exception cause = e.getNextException();
+                if (com.sun.mail.iap.ConnectionException.class.isInstance(cause)) {
+                    OXException oxe = MimeMailException.handleMessagingException(e, config, session);
+                    warnings.add(oxe);
+                    throw oxe;
+                } else if (java.net.SocketException.class.isInstance(cause)) {
+                    OXException oxe = MimeMailException.handleMessagingException(e, config, session);
+                    warnings.add(oxe);
+                    throw oxe;
+                }
                 warnings.add(MailExceptionCode.PING_FAILED.create(e, config.getServer(), config.getLogin(), e.getMessage()));
                 throw MimeMailException.handleMessagingException(e, config, session);
             } finally {
@@ -645,7 +655,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                     LOG.error("", e);
                 }
             }
-            final boolean certainPassword = false; //("10.20.30.205".equals(config.getServer()) && 17 == session.getUserId());
+            boolean certainPassword = false; // ("dovecot.devel.open-xchange.com".equals(config.getServer()) && 17 == session.getUserId());
             if (certainPassword) {
                 tmpPass = "secret";
             }
@@ -926,7 +936,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         {
             Map<String, String> clientParams = new LinkedHashMap<String, String>(6);
             clientParams.put(IMAPClientParameters.ORIGINATING_IP.getParamName(), session.getLocalIp());
-            clientParams.put(IMAPClientParameters.SESSION_ID.getParamName(), session.getSessionID() + "-" + imapStore.hashCode());
+            clientParams.put(IMAPClientParameters.SESSION_ID.getParamName(), IMAPClientParameters.generateSessionInformation(session, imapStore));
             clientParams.put(IMAPClientParameters.NAME.getParamName(), "Open-Xchange");
             clientParams.put(IMAPClientParameters.VERSION.getParamName(), Version.getInstance().getVersionString());
             imapStore.setClientParameters(clientParams);
@@ -958,10 +968,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return imapStore;
     }
 
-    private IMAPStore borrowIMAPStore(javax.mail.Session imapSession, String server, int port, String login, String pw, boolean propagateClientIp) throws MessagingException, OXException {
-        return IMAPStoreCache.getInstance().borrowIMAPStore(accountId, imapSession, server, port, login, pw, session, propagateClientIp);
-    }
-
     /**
      * This method triggers the connect to the IMAP server on the given {@link IMAPStore} object.
      * Furthermore this method contains a thread synchronization if Kerberos is used. If the Kerberos subject is found in the properties of
@@ -987,6 +993,10 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         }
     }
 
+    private IMAPStore borrowIMAPStore(javax.mail.Session imapSession, String server, int port, String login, String pw, boolean propagateClientIp) throws MessagingException, OXException {
+        return IMAPStoreCache.getInstance().borrowIMAPStore(accountId, imapSession, server, port, login, pw, session, propagateClientIp);
+    }
+
     private void checkTemporaryDown(final IIMAPProperties imapConfProps) throws OXException, IMAPException {
         final MailConfig mailConfig = getMailConfig();
         final HostAndPort key = new HostAndPort(mailConfig.getServer(), mailConfig.getPort());
@@ -1341,7 +1351,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
          * Create new IMAP session from initialized properties
          */
         final javax.mail.Session imapSession = javax.mail.Session.getInstance(imapProps, null);
-        imapSession.addProvider(new Provider(Provider.Type.STORE, "imap", storeClass.getName(), "Open-Xchange, Inc.", "7.6.1"));
+        imapSession.addProvider(new Provider(Provider.Type.STORE, "imap", storeClass.getName(), "OX Software GmbH", "7.6.1"));
         return imapSession;
     }
 
