@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,9 @@
 
 package com.openexchange.imap;
 
+import com.openexchange.session.Session;
+import com.sun.mail.imap.IMAPStore;
+
 
 /**
  * {@link IMAPClientParameters} - An enumeration for IMAP client parameters passed to IMAP store using <code>"ID"</code> command (if supported)
@@ -90,4 +93,28 @@ public enum IMAPClientParameters {
         return paramName;
     }
 
+    // --------------------------------------------------------------------------------------------------------------------
+
+    /**
+     * Generates the session information.
+     * <pre>
+     *  &lt;session-id&gt; + "-" &lt;user-id&gt; + "-" + &lt;context-id&gt; + "-" + &lt;store-hash&gt;
+     *
+     *  Example:
+     *  6ceec6585485458eb27456ad6ec97b62-17-1337-1356782
+     * </pre>
+     *
+     * @param session The user-associated session
+     * @param imapStore The IMAP store
+     * @return The session information
+     */
+    public static String generateSessionInformation(Session session, IMAPStore imapStore) {
+        StringBuilder buf = new StringBuilder(64);
+        buf.append(session.getSessionID());
+        buf.append('-').append(session.getUserId());
+        buf.append('-').append(session.getContextId());
+        buf.append('-').append(imapStore.hashCode());
+        return buf.toString();
+    }
+
 }
