@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.imap.cache;
 
+import static com.openexchange.imap.IMAPMessageStorage.allowSORTDISPLAY;
 import java.util.Map;
 import javax.mail.MessagingException;
 import com.openexchange.caching.CacheKey;
@@ -194,7 +195,8 @@ public final class CapabilitiesCache {
             imapCaps.setThreadReferences(map.containsKey(IMAPCapabilities.CAP_THREAD_REFERENCES));
             imapCaps.setThreadOrderedSubject(map.containsKey(IMAPCapabilities.CAP_THREAD_ORDEREDSUBJECT));
             imapCaps.setQuota(map.containsKey(IMAPCapabilities.CAP_QUOTA));
-            imapCaps.setSort(map.containsKey(IMAPCapabilities.CAP_SORT));
+            boolean hasSort = map.containsKey(IMAPCapabilities.CAP_SORT);
+            imapCaps.setSort(hasSort);
             imapCaps.setIMAP4(map.containsKey(IMAPCapabilities.CAP_IMAP4));
             imapCaps.setIMAP4rev1(map.containsKey(IMAPCapabilities.CAP_IMAP4_REV1));
             imapCaps.setUIDPlus(map.containsKey(IMAPCapabilities.CAP_UIDPLUS));
@@ -202,6 +204,13 @@ public final class CapabilitiesCache {
             imapCaps.setIdle(map.containsKey(IMAPCapabilities.CAP_IDLE));
             imapCaps.setChildren(map.containsKey(IMAPCapabilities.CAP_CHILDREN));
             imapCaps.setHasSubscription(!MailProperties.getInstance().isIgnoreSubscription());
+            if (hasSort && imapConfig.getIMAPProperties().isImapSort()) {
+                // IMAP sort supported & enabled
+                imapCaps.setSortDisplay(allowSORTDISPLAY(accontId) && map.containsKey(IMAPCapabilities.CAP_SORT_DISPLAY));
+            } else {
+                // The in-memory sorting does sort with primary respect to display name, the actual address
+                imapCaps.setSortDisplay(true);
+            }
             /*
              * ACL extension
              */
