@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.imap.command;
 
+import gnu.trove.impl.Constants;
 import gnu.trove.map.hash.TIntIntHashMap;
 import gnu.trove.map.hash.TLongIntHashMap;
 import java.io.IOException;
@@ -84,6 +85,7 @@ import com.openexchange.mail.mime.MimeMailException;
 import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.mime.converters.MimeMessageConverter;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
+import com.openexchange.mail.mime.utils.MimeStorageUtility;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.BODY;
@@ -124,6 +126,9 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
     private int index;
     private final MailMessage[] retval;
     private boolean determineAttachmentByHeader;
+    private boolean checkICal;
+    private boolean checkVCard;
+    private boolean treatEmbeddedAsAttachment;
     private final String fullname;
     private final Set<FetchItemHandler> lastHandlers;
     private final TLongIntHashMap uid2index;
@@ -139,7 +144,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * @param fp The fetch profile to use
      * @throws MessagingException If initialization fails
      */
-    public MailMessageFetchIMAPCommand(final IMAPFolder imapFolder, final char separator, final boolean isRev1, final int[] seqNums, final FetchProfile fp) throws MessagingException {
+    public MailMessageFetchIMAPCommand(IMAPFolder imapFolder, char separator, boolean isRev1, int[] seqNums, FetchProfile fp) throws MessagingException {
         super(imapFolder);
         determineAttachmentByHeader = false;
         final int messageCount = imapFolder.getMessageCount();
@@ -151,7 +156,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         command = getFetchCommand(isRev1, fp, false);
         uid = false;
         length = seqNums.length;
-        seqNum2index = new TIntIntHashMap(length);
+        seqNum2index = new TIntIntHashMap(length, Constants.DEFAULT_LOAD_FACTOR, 0, -1);
         uid2index = null;
         for (int i = 0; i < length; i++) {
             seqNum2index.put(seqNums[i], i);
@@ -164,6 +169,37 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         retval = new MailMessage[length];
     }
 
+    /**
+     * Initializes a new {@link MailMessageFetchIMAPCommand} to fetch all messages.
+     *
+     * @param imapFolder The IMAP folder providing connected protocol
+     * @param separator The separator character
+     * @param isRev1 Whether IMAP server has <i>IMAP4rev1</i> capability or not
+     * @param fp The fetch profile to use
+     * @throws MessagingException If initialization fails
+     */
+    public MailMessageFetchIMAPCommand(IMAPFolder imapFolder, char separator, boolean isRev1, FetchProfile fp) throws MessagingException {
+        super(imapFolder);
+        determineAttachmentByHeader = false;
+        final int messageCount = imapFolder.getMessageCount();
+        if (messageCount <= 0) {
+            returnDefaultValue = true;
+        }
+        this.separator = separator;
+        lastHandlers = new HashSet<FetchItemHandler>();
+        command = getFetchCommand(isRev1, fp, false);
+        uid = false;
+        length = messageCount;
+        uid2index = null;
+        seqNum2index = null;
+        args = (1 == length ? new String[] { "1" } : ARGS_ALL);
+        if (0 == length) {
+            returnDefaultValue = true;
+        }
+        fullname = imapFolder.getFullName();
+        retval = new MailMessage[length];
+    }
+
     /**
      * Initializes a new {@link MailMessageFetchIMAPCommand}.
      *
@@ -174,7 +210,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * @param fp The fetch profile to use
      * @throws MessagingException If initialization fails
      */
-    public MailMessageFetchIMAPCommand(final IMAPFolder imapFolder, final char separator, final boolean isRev1, final long[] uids, final FetchProfile fp) throws MessagingException {
+    public MailMessageFetchIMAPCommand(IMAPFolder imapFolder, char separator, boolean isRev1, long[] uids, FetchProfile fp) throws MessagingException {
         super(imapFolder);
         determineAttachmentByHeader = false;
         final int messageCount = imapFolder.getMessageCount();
@@ -184,7 +220,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         this.separator = separator;
         lastHandlers = new HashSet<FetchItemHandler>();
         length = uids.length;
-        uid2index = new TLongIntHashMap(length);
+        uid2index = new TLongIntHashMap(length, Constants.DEFAULT_LOAD_FACTOR, 0, -1);
         seqNum2index = null;
         for (int i = 0; i < length; i++) {
             uid2index.put(uids[i], i);
@@ -215,11 +251,44 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      *            only; otherwise <code>false</code>
      * @return This FETCH IMAP command with value applied
      */
-    public MailMessageFetchIMAPCommand setDetermineAttachmentByHeader(final boolean determineAttachmentByHeader) {
+    public MailMessageFetchIMAPCommand setDetermineAttachmentByHeader(boolean determineAttachmentByHeader) {
         this.determineAttachmentByHeader = determineAttachmentByHeader;
         return this;
     }
 
+    /**
+     * Sets the checkICal
+     *
+     * @param checkICal The checkICal to set
+     * @return This FETCH IMAP command with value applied
+     */
+    public MailMessageFetchIMAPCommand setCheckICal(boolean checkICal) {
+        this.checkICal = checkICal;
+        return this;
+    }
+
+    /**
+     * Sets the checkVCard
+     *
+     * @param checkVCard The checkVCard to set
+     * @return This FETCH IMAP command with value applied
+     */
+    public MailMessageFetchIMAPCommand setCheckVCard(boolean checkVCard) {
+        this.checkVCard = checkVCard;
+        return this;
+    }
+
+    /**
+     * Sets the treatEmbeddedAsAttachment
+     *
+     * @param treatEmbeddedAsAttachment The treatEmbeddedAsAttachment to set
+     * @return This FETCH IMAP command with value applied
+     */
+    public MailMessageFetchIMAPCommand setTreatEmbeddedAsAttachment(boolean treatEmbeddedAsAttachment) {
+        this.treatEmbeddedAsAttachment = treatEmbeddedAsAttachment;
+        return this;
+    }
+
     @Override
     protected String getDebugInfo(final int argsIndex) {
         final StringBuilder sb = new StringBuilder(command.length() + 64);
@@ -303,6 +372,9 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
     }
 
     private IDMailMessage handleMessage(final int seqNum) {
+        if (seqNum < 0) {
+            return null;
+        }
         try {
             return handleMessage(imapFolder.getMessage(seqNum));
         } catch (final Exception e) {
@@ -312,6 +384,9 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
     }
 
     private IDMailMessage handleMessage(final long uid) {
+        if (uid < 0) {
+            return null;
+        }
         try {
             return handleMessage(imapFolder.getMessageByUID(uid));
         } catch (final Exception e) {
@@ -346,26 +421,30 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         }
         final FetchResponse fetchResponse = (FetchResponse) currentReponse;
         final int seqNum = fetchResponse.getNumber();
-        final int pos;
-        if (null == seqNum2index) {
-            final UID uidItem = getItemOf(UID.class, fetchResponse);
-            if (null != uidItem && uid2index.containsKey(uidItem.uid)) {
+        int pos;
+        if (null != seqNum2index) {
+            pos = seqNum2index.remove(seqNum);
+            if (pos < 0) {
+                pos = index;
+            }
+        } else if (null != uid2index) {
+            UID uidItem = getItemOf(UID.class, fetchResponse);
+            if (null != uidItem) {
                 pos = uid2index.remove(uidItem.uid);
+                if (pos < 0) {
+                    pos = index;
+                }
             } else {
                 pos = index;
             }
         } else {
-            if (seqNum2index.containsKey(seqNum)) {
-                pos = seqNum2index.remove(seqNum);
-            } else {
-                pos = index;
-            }
+            pos = index;
         }
         index++;
         boolean error = false;
         MailMessage mail;
         try {
-            mail = handleFetchRespone(fetchResponse, fullname, separator, lastHandlers, determineAttachmentByHeader);
+            mail = handleFetchRespone(fetchResponse, fullname, separator, lastHandlers, determineAttachmentByHeader, checkICal, checkVCard, treatEmbeddedAsAttachment);
         } catch (final MessagingException e) {
             /*
              * Discard corrupt message
@@ -401,7 +480,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * @throws OXException If an OX error occurs
      */
     public static MailMessage handleFetchRespone(final FetchResponse fetchResponse, final String fullName, final char separator) throws MessagingException, OXException {
-        return handleFetchRespone(new IDMailMessage(null, fullName), fetchResponse, fullName, separator, null, false);
+        return handleFetchRespone(new IDMailMessage(null, fullName), fetchResponse, fullName, separator, null, false, false, false, false);
     }
 
     /**
@@ -416,14 +495,14 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * @throws OXException If an OX error occurs
      */
     public static MailMessage handleFetchRespone(final IDMailMessage mail, final FetchResponse fetchResponse, final String fullName, final char separator) throws MessagingException, OXException {
-        return handleFetchRespone(mail, fetchResponse, fullName, separator, null, false);
+        return handleFetchRespone(mail, fetchResponse, fullName, separator, null, false, false, false, false);
     }
 
-    private static MailMessage handleFetchRespone(final FetchResponse fetchResponse, final String fullName, final char separator, final Set<FetchItemHandler> lastHandlers, final boolean determineAttachmentByHeader) throws MessagingException, OXException {
-        return handleFetchRespone(new IDMailMessage(null, fullName), fetchResponse, fullName, separator, lastHandlers, determineAttachmentByHeader);
+    private static MailMessage handleFetchRespone(FetchResponse fetchResponse, String fullName, char separator, Set<FetchItemHandler> lastHandlers, boolean determineAttachmentByHeader, boolean checkICal, boolean checkVCard, boolean treatEmbeddedAsAttachment) throws MessagingException, OXException {
+        return handleFetchRespone(new IDMailMessage(null, fullName), fetchResponse, fullName, separator, lastHandlers, determineAttachmentByHeader, checkICal, checkVCard, treatEmbeddedAsAttachment);
     }
 
-    private static MailMessage handleFetchRespone(final IDMailMessage mail, final FetchResponse fetchResponse, final String fullName, final char separator, final Set<FetchItemHandler> lastHandlers, final boolean determineAttachmentByHeader) throws MessagingException, OXException {
+    private static MailMessage handleFetchRespone(IDMailMessage mail, FetchResponse fetchResponse, String fullName, char separator, Set<FetchItemHandler> lastHandlers, boolean determineAttachmentByHeader, boolean checkICal, boolean checkVCard, boolean treatEmbeddedAsAttachment) throws MessagingException, OXException {
         final IDMailMessage m;
         if (null == mail) {
             m = new IDMailMessage(null, fullName);
@@ -442,7 +521,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             final Item item = fetchResponse.getItem(j);
             FetchItemHandler itemHandler = map.get(item.getClass());
             if (null == itemHandler) {
-                itemHandler = getItemHandlerByItem(item);
+                itemHandler = getItemHandlerByItem(item, checkICal, checkVCard, treatEmbeddedAsAttachment);
                 if (null == itemHandler) {
                     LOG.warn("Unknown FETCH item: {}", item.getClass().getName());
                 } else {
@@ -467,8 +546,10 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         return m;
     }
 
-    private static FetchItemHandler getItemHandlerByItem(final Item item) {
-        if ((item instanceof RFC822DATA) || (item instanceof BODY)) {
+    private static FetchItemHandler getItemHandlerByItem(Item item, boolean checkICal, boolean checkVCard, boolean treatEmbeddedAsAttachment) {
+        if (item instanceof BODYSTRUCTURE) {
+            return new BODYSTRUCTUREFetchItemHandler(checkICal, checkVCard, treatEmbeddedAsAttachment);
+        } else if ((item instanceof RFC822DATA) || (item instanceof BODY)) {
             return HEADER_ITEM_HANDLER;
         } else if (item instanceof UID) {
             return UID_ITEM_HANDLER;
@@ -480,12 +561,6 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             return ENVELOPE_ITEM_HANDLER;
         } else if (item instanceof RFC822SIZE) {
             return SIZE_ITEM_HANDLER;
-        } else if (item instanceof BODYSTRUCTURE) {
-            return BODYSTRUCTURE_ITEM_HANDLER;
-        } else if (item instanceof X_REAL_UID) {
-            return X_REAL_UID_ITEM_HANDLER;
-        } else if (item instanceof com.sun.mail.imap.protocol.X_MAILBOX) {
-            return X_MAILBOX_ITEM_HANDLER;
         } else {
             return null;
         }
@@ -645,7 +720,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             for (final Enumeration<?> e = h.getAllHeaders(); e.hasMoreElements();) {
                 final Header hdr = (Header) e.nextElement();
                 final String name = hdr.getName();
-                headerFields.remove(toLowerCase(name));
+                headerFields.remove(Strings.toLowerCase(name));
                 {
                     final HeaderHandler headerHandler = hh.get(name);
                     if (null != headerHandler) {
@@ -848,16 +923,18 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             msg.setSentDate(env.date);
         }
 
-        private InternetAddress[] wrap(final InternetAddress... addresses) {
+        private InternetAddress[] wrap(InternetAddress... addresses) {
             if (null == addresses || 0 == addresses.length) {
                 return null;
             }
-            final int length = addresses.length;
-            final InternetAddress[] ret = new InternetAddress[length];
-            for (int i = 0; i < length; i++) {
+
+            int length = addresses.length;
+            InternetAddress[] ret = new InternetAddress[length];
+            for (int i = length; i-- > 0;) {
                 try {
                     ret[i] = new QuotedInternetAddress(addresses[i].toString());
-                } catch (final AddressException e) {
+                } catch (AddressException e) {
+                    // Use as-is
                     ret[i] = addresses[i];
                 }
             }
@@ -913,22 +990,35 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         }
     };
 
-    private static final FetchItemHandler BODYSTRUCTURE_ITEM_HANDLER = new FetchItemHandler() {
+    // ------------------------------------------------------------------------------------------------------------------------------------------------
+
+    private static final class BODYSTRUCTUREFetchItemHandler implements FetchItemHandler {
+
+        private final boolean checkICal;
+        private final boolean checkVCard;
+        private final boolean treatEmbeddedAsAttachment;
+
+        BODYSTRUCTUREFetchItemHandler(boolean checkICal, boolean checkVCard, boolean treatEmbeddedAsAttachment) {
+            super();
+            this.checkICal = checkICal;
+            this.checkVCard = checkVCard;
+            this.treatEmbeddedAsAttachment = treatEmbeddedAsAttachment;
+        }
 
         @Override
-        public void handleItem(final Item item, final IDMailMessage msg, final org.slf4j.Logger logger) throws OXException {
-            final BODYSTRUCTURE bs = (BODYSTRUCTURE) item;
-            final ContentType contentType = new ContentType().setPrimaryType(bs.type).setSubType(bs.subtype);
+        public void handleItem(Item item, IDMailMessage msg, org.slf4j.Logger logger) throws OXException {
+            BODYSTRUCTURE bs = (BODYSTRUCTURE) item;
+            ContentType contentType = new ContentType().setPrimaryType(bs.type).setSubType(bs.subtype);
             {
-                final ParameterList cParams = bs.cParams;
+                ParameterList cParams = bs.cParams;
                 if (cParams != null) {
-                    for (final Enumeration<?> names = cParams.getNames(); names.hasMoreElements();) {
-                        final String name = names.nextElement().toString();
-                        final String value = cParams.get(name);
-                        if (!isEmpty(value)) {
+                    for (Enumeration<?> names = cParams.getNames(); names.hasMoreElements();) {
+                        String name = names.nextElement().toString();
+                        String value = cParams.get(name);
+                        if (!com.openexchange.java.Strings.isEmpty(value)) {
                             try {
-                                contentType.setParameterErrorAware(name, value);
-                            } catch (final OXException e) {
+                                contentType.setParameterErrorAware(name, MimeMessageUtility.decodeEnvelopeHeader(value));
+                            } catch (OXException e) {
                                 final long uid = msg.getUid();
                                 final String folder = msg.getFolder();
                                 LOG.debug("Ignoring invalid parameter in Content-Type header contained in message {} of folder {}.", uid <= 0 ? "<unknown>" : Long.toString(uid), null == folder ? "<unknown>" : folder);
@@ -939,7 +1029,73 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             }
             msg.setContentType(contentType);
             msg.addHeader("Content-Type", contentType.toString(true));
-            msg.setHasAttachment(bs.isMulti() && (MULTI_SUBTYPE_MIXED.equalsIgnoreCase(bs.subtype) || MimeMessageUtility.hasAttachments(bs)));
+
+            boolean hasAttachment = bs.isMulti() && (MULTI_SUBTYPE_MIXED.equalsIgnoreCase(bs.subtype) || MimeMessageUtility.hasAttachments(bs));
+            if (hasAttachment) {
+                if (checkICal && hasICal(bs)) {
+                    msg.addHeader("X-ICAL", "true");
+                }
+
+                if (checkVCard && hasVCard(bs)) {
+                    msg.addHeader("X-VCARD", "true");
+                }
+            } else if (treatEmbeddedAsAttachment) {
+                hasAttachment = hasEmbedded(bs);
+            }
+            msg.setHasAttachment(hasAttachment);
+        }
+
+        private boolean hasICal(BODYSTRUCTURE bs) {
+            String baseContentType = bs.type + "/" + bs.subtype;
+            if (baseContentType.indexOf("calendar") >= 0 || baseContentType.indexOf("ics") >= 0) {
+                return true;
+            }
+
+            BODYSTRUCTURE[] subs = bs.bodies;
+            if (null != subs) {
+                for (BODYSTRUCTURE sub : subs) {
+                    if (hasICal(sub)) {
+                        return true;
+                    }
+                }
+            }
+
+            return false;
+        }
+
+        private boolean hasVCard(BODYSTRUCTURE bs) {
+            String baseContentType = bs.type + "/" + bs.subtype;
+            if (baseContentType.indexOf("card") >= 0 || baseContentType.indexOf("vcf") >= 0) {
+                return true;
+            }
+
+            BODYSTRUCTURE[] subs = bs.bodies;
+            if (null != subs) {
+                for (BODYSTRUCTURE sub : subs) {
+                    if (hasVCard(sub)) {
+                        return true;
+                    }
+                }
+            }
+
+            return false;
+        }
+
+        private boolean hasEmbedded(BODYSTRUCTURE bs) {
+            if ("image".equals(bs.type) && !Strings.isEmpty(bs.id)) {
+                return true;
+            }
+
+            BODYSTRUCTURE[] subs = bs.bodies;
+            if (null != subs) {
+                for (BODYSTRUCTURE sub : subs) {
+                    if (hasEmbedded(sub)) {
+                        return true;
+                    }
+                }
+            }
+
+            return false;
         }
 
         @Override
@@ -968,63 +1124,36 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
                 }
             }
         }
-    };
+    }
+
+    // ------------------------------------------------------------------------------------------------------------------------------------------------
 
     private static final FetchItemHandler UID_ITEM_HANDLER = new FetchItemHandler() {
 
         @Override
         public void handleItem(final Item item, final IDMailMessage msg, final org.slf4j.Logger logger) {
-            final long id = ((UID) item).uid;
+            long id = ((UID) item).uid;
             msg.setMailId(Long.toString(id));
             msg.setUid(id);
         }
 
         @Override
         public void handleMessage(final Message message, final IDMailMessage msg, final org.slf4j.Logger logger) throws MessagingException {
-            final long id = ((IMAPFolder) message.getFolder()).getUID(message);
+            long id = ((IMAPFolder) message.getFolder()).getUID(message);
             msg.setMailId(Long.toString(id));
             msg.setUid(id);
         }
     };
 
-    private static final FetchItemHandler X_REAL_UID_ITEM_HANDLER = new FetchItemHandler() {
-
-        @Override
-        public void handleItem(final Item item, final IDMailMessage msg, final org.slf4j.Logger logger) {
-            msg.setUid(((X_REAL_UID) item).uid);
-        }
-
-        @Override
-        public void handleMessage(final Message message, final IDMailMessage msg, final org.slf4j.Logger logger) throws MessagingException {
-            // Nothing
-        }
-    };
-
-    private static final FetchItemHandler X_MAILBOX_ITEM_HANDLER = new FetchItemHandler() {
-
-        @Override
-        public void handleItem(final Item item, final IDMailMessage msg, final org.slf4j.Logger logger) {
-            msg.setFolder(((com.sun.mail.imap.protocol.X_MAILBOX) item).mailbox);
-        }
-
-        @Override
-        public void handleMessage(final Message message, final IDMailMessage msg, final org.slf4j.Logger logger) throws MessagingException {
-            // Nothing
-        }
-    };
-
     private static final Map<Class<? extends Item>, FetchItemHandler> MAP;
 
     static {
         MAP = new HashMap<Class<? extends Item>, FetchItemHandler>(8);
         MAP.put(UID.class, UID_ITEM_HANDLER);
-        MAP.put(X_REAL_UID.class, X_REAL_UID_ITEM_HANDLER);
-        MAP.put(com.sun.mail.imap.protocol.X_MAILBOX.class, X_MAILBOX_ITEM_HANDLER);
         MAP.put(INTERNALDATE.class, INTERNALDATE_ITEM_HANDLER);
         MAP.put(FLAGS.class, FLAGS_ITEM_HANDLER);
         MAP.put(ENVELOPE.class, ENVELOPE_ITEM_HANDLER);
         MAP.put(RFC822SIZE.class, SIZE_ITEM_HANDLER);
-        MAP.put(BODYSTRUCTURE.class, BODYSTRUCTURE_ITEM_HANDLER);
         MAP.put(INTERNALDATE.class, INTERNALDATE_ITEM_HANDLER);
     }
 
@@ -1032,19 +1161,6 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * ++++++++++++++ End of item handlers ++++++++++++++
      */
 
-    public static final class FetchItem extends FetchProfile.Item {
-
-        /**
-         * Initializes a new {@link FetchItem}.
-         *
-         * @param name The name
-         */
-        protected FetchItem(String name) {
-            super(name);
-        }
-
-    }
-
     /**
      * This is the Envelope item. Despite of JavaMail's ENVELOPE item, this item does not include INTERNALDATE nor RFC822.SIZE; it solely
      * consists of the ENVELOPE.
@@ -1052,17 +1168,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * The Envelope is an aggregation of the common attributes of a Message. Implementations should include the following attributes: From,
      * To, Cc, Bcc, ReplyTo, Subject and Date. More items may be included as well.
      */
-    public static final FetchProfile.Item ENVELOPE_ONLY = new FetchItem("ENVELOPE_ONLY");
-
-    /**
-     * This is the X-MAILBOX item.
-     */
-    public static final FetchProfile.Item X_MAILBOX = new FetchItem("X-MAILBOX");
-
-    /**
-     * This is the X-REAL-UID item.
-     */
-    public static final FetchProfile.Item X_REAL_UID = new FetchItem("X-REAL-UID");
+    public static final FetchProfile.Item ENVELOPE_ONLY = new MimeStorageUtility.FetchItem("ENVELOPE_ONLY");
 
     /**
      * Turns given fetch profile into FETCH items to craft a FETCH command.
@@ -1072,9 +1178,9 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
      * @param loadBody <code>true</code> if message body should be loaded; otherwise <code>false</code>
      * @return The FETCH items to craft a FETCH command
      */
-    public static String getFetchCommand(final boolean isRev1, final FetchProfile fp, final boolean loadBody) {
-        final StringBuilder command = new StringBuilder(128);
-        final boolean sizeIncluded;
+    public static String getFetchCommand(boolean isRev1, FetchProfile fp, boolean loadBody) {
+        StringBuilder command = new StringBuilder(128);
+        boolean sizeIncluded;
         if (fp.contains(FetchProfile.Item.ENVELOPE)) {
             if (loadBody) {
                 command.append("INTERNALDATE");
@@ -1100,25 +1206,20 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         if (fp.contains(FetchProfile.Item.CONTENT_INFO)) {
             command.append(" BODYSTRUCTURE");
         }
-        if (fp.contains(UIDFolder.FetchProfileItem.UID)) {
+        boolean uidIncluded = fp.contains(UIDFolder.FetchProfileItem.UID);
+        if (uidIncluded) {
             command.append(" UID");
         }
-        if (fp.contains(X_MAILBOX)) {
-            command.append(" X-MAILBOX");
-        }
-        if (fp.contains(X_REAL_UID)) {
-            command.append(" X-REAL-UID");
-        }
-        boolean allHeaders = false;
-        if (fp.contains(IMAPFolder.FetchProfileItem.HEADERS) && !loadBody) {
-            allHeaders = true;
+
+        boolean allHeaders = (fp.contains(IMAPFolder.FetchProfileItem.HEADERS) && !loadBody);
+        if (allHeaders) {
             if (isRev1) {
                 command.append(" BODY.PEEK[HEADER]");
             } else {
                 command.append(" RFC822.HEADER");
             }
         }
-        if (!sizeIncluded && fp.contains(IMAPFolder.FetchProfileItem.SIZE)) {
+        if (!sizeIncluded && fp.contains(FetchProfile.Item.SIZE)) {
             command.append(" RFC822.SIZE");
         }
         /*
@@ -1204,18 +1305,4 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
         }
         return null;
     }
-
-    /** Check for an empty string */
-    protected static boolean isEmpty(final String string) {
-        if (null == string) {
-            return true;
-        }
-        final int len = string.length();
-        boolean isWhitespace = true;
-        for (int i = 0; isWhitespace && i < len; i++) {
-            isWhitespace = Strings.isWhitespace(string.charAt(i));
-        }
-        return isWhitespace;
-    }
-
 }
