@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -75,6 +75,7 @@ import com.openexchange.mail.mime.MimeMailException;
 import com.openexchange.mail.mime.converters.MimeMessageConverter;
 import com.openexchange.mail.mime.dataobjects.MimeRawSource;
 import com.openexchange.mail.mime.datasource.MessageDataSource;
+import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.sun.mail.iap.ByteArray;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPInputStream;
@@ -234,7 +235,7 @@ public final class IMAPMailPart extends MailPart implements MimeRawSource, Conne
                 if (null != dParams) {
                     for (final Enumeration<?> names = dParams.getNames(); names.hasMoreElements();) {
                         final String name = names.nextElement().toString();
-                        contentDisposition.setParameter(name, dParams.get(name));
+                        contentDisposition.setParameter(name, MimeMessageUtility.decodeEnvelopeHeader(dParams.get(name)));
                     }
                 }
                 setContentDisposition(contentDisposition);
@@ -253,7 +254,7 @@ public final class IMAPMailPart extends MailPart implements MimeRawSource, Conne
                 if (null != cParams) {
                     for (final Enumeration<?> names = cParams.getNames(); names.hasMoreElements();) {
                         final String name = names.nextElement().toString();
-                        contentType.setParameter(name, cParams.get(name));
+                        contentType.setParameter(name, MimeMessageUtility.decodeEnvelopeHeader(cParams.get(name)));
                     }
                 }
                 setContentType(contentType);
